//------------------------------------------------------------------------------
// Расчет температуры кипения и мольной концентрации этанола в паре при заданных
// давлении и мольной концентрациии этанола в жидкой фазе (раствор этанола в воде).
// Коэффициенты активности рассчитываются в приближении Ван-Лаара (van Laar).
// Вариант для Arduino. Проверено на Arduino UNO.
//
// Использование:
// После загрузки скетча в мониторе последовательного порта набрать давление
// (просто число) в мм.рт.ст. и ввод. Через несколько секунд скетч выдаст
// табличку, содержащая:
// мольную концентрацию спирта в жидкой фазе,
// температуру кипения в градусах Цельсия и
// мольную концентрацию этанола.
//------------------------------------------------------------------------------
// 16.12.18  OldBean
//------------------------------------------------------------------------------
// Давление насыщенного пара - по уравнению Антуан
static float cA1 = 8.21330, cB1 = 1652.050, cC1 = 231.480; // Этанол
static float cA2 = 8.07131, cB2 = 1730.63, cC2 = 233.426; // Вода
float Psat1(float T) {
  return powf(10.0, cA1 - cB1/(T + cC1));
}
float Psat2(float T) {
  return powf(10.0, cA2 - cB2/(T + cC2));
}
//------------------------------------------------------------------------------
// Коэффициенты активности по Ван Лаару (van Laar)
# define M_E 2.7182818284590452354
float cA12 = 1.6798, cA21 = 0.9227;
float Gam1(float xa) { // Этанол
  float wrk = cA21*(1.0 - xa)/(cA12*xa + cA21*(1.0 - xa));
  return powf(M_E, cA12*wrk*wrk);
}
float Gam2(float xa) { // Вода
  float wrk = cA12*xa/(cA21*(1.0 - xa) + cA12*xa);
  return powf(M_E, cA21*wrk*wrk);
}
//------------------------------------------------------------------------------
float alpha = 0, beta = 0; // Это xa*Gam1(xa) и (1.0 - xa)*Gam2(xa). 
// Эти глобальные переменные нужны просто для ускорения итерационного процесса 
// по T, где xa уже не изменяется.
float P = 760.0; // Давление
//------------------------------------------------------------------------------
// Модифицированный закон Рауля (с активностями)
// Корень этой функции дает температуру кипения раствора
float Raoult(float T) {
  return Psat1(T)*alpha + Psat2(T)*beta - P; 
}
//------------------------------------------------------------------------------
// Вычисление корня методом дихотомии
float my_root(float aa, float bb, float eps, float its) {
  float a, b;
  if(aa < bb) {a = aa; b = bb;}
  else {a = bb; b = aa;}
  float fa = Raoult(a), fb = Raoult(b);
  if(fa*fb > 0) {
    fprintf(stderr, "В заданном интервале нет корней (или больше одного корня)\n");
    return 0;
  }
  int it = 0; 
  float dx = b - a;
  while(it < its && dx > eps) {
    float c = 0.5*(a + b); 
    float fc = Raoult(c);
    if(fa*fc >= 0) {a = c; fa = fc;}
    else {b = c; fb = fc;}
    dx = b - a; it += 1;
  }
  return 0.5*(a + b);
}
//------------------------------------------------------------------------------
void setup() {
  Serial.begin(9600); // Инициализация последовательного порта
}
//------------------------------------------------------------------------------
void loop() {
  P = Serial.parseFloat();
  if(P > 0) {
    Serial.print("P = ");
    Serial.print(P);
    Serial.println(" mmHg");
    Serial.println("x,mole/mole\tT,C\ty,mole/mole");
    for(int i = 0; i < 51; i++) {
      float xa = i*0.02; // Мольная концентрация этанола в жидкой фазе
      alpha = xa*Gam1(xa); beta = (1.0 - xa)*Gam2(xa);
      float Tb = my_root(0.0, 150.0, 1.0E-6, 30);
      float ya = Psat1(Tb)*xa*Gam1(xa)/P;
      Serial.print(xa, 3); Serial.print("\t");
      Serial.print(Tb, 3); Serial.print("\t");
      Serial.println(ya, 3);
    }
  }
}
//------------------------------------------------------------------------------
